/*
CallListener.java
Copyright (C) 2010  Belledonne Communications, Grenoble, France

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

package org.linphone.core;

import androidx.annotation.Nullable;
import androidx.annotation.NonNull;

/**
* That class holds all the callbacks which are called by LinphoneCall objects. <br/>
* <br/>
* Use linphone_factory_create_call_cbs to create an instance. Then, call the<br/>
* callback setters on the events you need to monitor and pass the object to a<br/>
* LinphoneCall instance through linphone_call_add_callbacks. <br/>
*/
public interface CallListener {
    /**
    * Call state notification callback. <br/>
    * <br/>
    * @param call LinphoneCall whose state is changed.   <br/>
    * @param state The new LinphoneCallState of the call <br/>
    * @param message An informational message about the state.   <br/>
    */
    public void onStateChanged(@NonNull Call call, Call.State state, @NonNull String message);

    /**
    * Callback to notify that the call is being recorded by the remote. <br/>
    * <br/>
    * @param call LinphoneCall for which the audio is recorded   <br/>
    * @param recording true if the call is being recorded by the remote, false<br/>
    * otherwise <br/>
    */
    public void onRemoteRecording(@NonNull Call call, boolean recording);

    /**
    * Callback to notify a next video frame has been decoded. <br/>
    * <br/>
    * @param call LinphoneCall for which the next video frame has been decoded   <br/>
    */
    public void onNextVideoFrameDecoded(@NonNull Call call);

    /**
    * Callback for notifying a received TMMBR. <br/>
    * <br/>
    * @param call LinphoneCall for which the TMMBR has changed   <br/>
    * @param streamIndex the index of the current stream <br/>
    * @param tmmbr the value of the received TMMBR <br/>
    */
    public void onTmmbrReceived(@NonNull Call call, int streamIndex, int tmmbr);

    /**
    * Call encryption changed callback. <br/>
    * <br/>
    * @param call LinphoneCall object whose encryption is changed.   <br/>
    * @param on Whether encryption is activated. <br/>
    * @param authenticationToken An authentication_token, currently set for ZRTP kind<br/>
    * of encryption only.   <br/>
    */
    public void onEncryptionChanged(@NonNull Call call, boolean on, @Nullable String authenticationToken);

    /**
    * Callback for receiving info messages. <br/>
    * <br/>
    * @param call LinphoneCall whose info message belongs to.   <br/>
    * @param message LinphoneInfoMessage object.   <br/>
    */
    public void onInfoMessageReceived(@NonNull Call call, @NonNull InfoMessage message);

    /**
    * Callback for being notified of received DTMFs. <br/>
    * <br/>
    * @param call LinphoneCall object that received the dtmf   <br/>
    * @param dtmf The ascii code of the dtmf <br/>
    */
    public void onDtmfReceived(@NonNull Call call, int dtmf);

    /**
    * Callback to notify that the camera is not working and has been changed to "No<br/>
    * Webcam". <br/>
    * <br/>
    * A camera is detected as mis-functionning as soon as it outputs no frames at all<br/>
    * during a period of 5 seconds. This check is only performed on desktop<br/>
    * platforms, in the purpose of notifying camera failures, for example if when a<br/>
    * usb cable gets disconnected.<br/>
    * @param call LinphoneCall for which the next video frame has been decoded   <br/>
    * @param cameraName the name of the non-working camera   <br/>
    */
    public void onCameraNotWorking(@NonNull Call call, @NonNull String cameraName);

    /**
    * Callback to notify that the audio device has been changed. <br/>
    * <br/>
    * @param call LinphoneCall for which the audio device has changed   <br/>
    * @param audioDevice the new audio device used for this call   <br/>
    */
    public void onAudioDeviceChanged(@NonNull Call call, @NonNull AudioDevice audioDevice);

    /**
    * Callback for receiving quality statistics for calls. <br/>
    * <br/>
    * @param call LinphoneCall object whose statistics are notified   <br/>
    * @param stats LinphoneCallStats object   <br/>
    */
    public void onStatsUpdated(@NonNull Call call, @NonNull CallStats stats);

    /**
    * Callback for notifying progresses of transfers. <br/>
    * <br/>
    * @param call LinphoneCall that was transfered   <br/>
    * @param state The LinphoneCallState of the call to transfer target at the far<br/>
    * end. <br/>
    */
    public void onTransferStateChanged(@NonNull Call call, Call.State state);

    /**
    * Callback for notifying the processing SIP ACK messages. <br/>
    * <br/>
    * @param call LinphoneCall for which an ACK is being received or sent   <br/>
    * @param ack the ACK LinphoneHeaders   <br/>
    * @param isReceived if true this ACK is an incoming one, otherwise it is an ACK<br/>
    * about to be sent. <br/>
    */
    public void onAckProcessing(@NonNull Call call, @NonNull Headers ack, boolean isReceived);

    /**
    * Callback for notifying a snapshot taken. <br/>
    * <br/>
    * @param call LinphoneCall for which the snapshot was taken   <br/>
    * @param filePath the name of the saved file   <br/>
    */
    public void onSnapshotTaken(@NonNull Call call, @NonNull String filePath);

}